import type { ODataSelect, ODataSelectResult } from './lib/odata';
import type { FindDocumentRequest, PostDocumentsRequest, SearchDocumentsPageResult, SearchDocumentsRequest, SuggestDocumentsResult, Suggester, SuggestRequest, AutoCompleteRequest, AutoCompleteDocumentResult, Schema, ScoringProfile } from './services';
export declare class Index<T extends object> {
    readonly name: string;
    private dataStore;
    private searchEngine;
    private suggestEngine;
    private autocompleteEngine;
    static createIndex<T extends object>(options: {
        name: string;
        schema: Schema;
        suggesters?: Suggester[];
        scoringProfiles?: ScoringProfile<T>[];
        defaultScoringProfile?: string;
    }): Index<T>;
    private constructor();
    /**
     * https://learn.microsoft.com/en-us/rest/api/searchservice/addupdate-or-delete-documents
     * @param documents
     */
    postDocuments(documents: PostDocumentsRequest<T>): void;
    /**
     * https://learn.microsoft.com/en-us/rest/api/searchservice/lookup-document
     * @param request
     */
    findDocument<Keys extends ODataSelect<T>>(request: FindDocumentRequest<T, Keys>): ODataSelectResult<T, Keys>;
    /**
     * https://learn.microsoft.com/en-us/rest/api/searchservice/count-documents
     */
    countDocuments(): number;
    /**
     * https://learn.microsoft.com/en-us/rest/api/searchservice/search-documents
     * @param request
     */
    search<Keys extends ODataSelect<T>>(request: SearchDocumentsRequest<T, Keys>): SearchDocumentsPageResult<ODataSelectResult<T, Keys>>;
    /**
     * https://learn.microsoft.com/en-us/rest/api/searchservice/autocomplete
     * @param request
     */
    autocomplete(request: AutoCompleteRequest): AutoCompleteDocumentResult;
    /**
     * https://learn.microsoft.com/en-us/rest/api/searchservice/suggestions
     * @param request
     */
    suggest<Keys extends ODataSelect<T>>(request: SuggestRequest<T, Keys>): SuggestDocumentsResult<ODataSelectResult<T, Keys>>;
}
export declare class Emulator {
    private indices;
    /**
     * https://learn.microsoft.com/en-us/rest/api/searchservice/create-index
     */
    createIndex<T extends object>(options: {
        name: string;
        schema: Schema;
        suggesters?: Suggester[];
        scoringProfiles?: ScoringProfile<T>[];
        defaultScoringProfile?: string;
    }): Index<T>;
    getIndex<T extends object>(name: string): Index<T>;
}
