import { GeoJSONPoint } from './geo';
export declare type UnionToIntersection<U> = (U extends unknown ? (_: U) => unknown : never) extends (_: infer I) => unknown ? I : never;
export declare type ExcludedODataTypes = Date | GeoJSONPoint;
/**
 * Produces a union of valid Cognitive Search OData $select paths for T
 * using a post-order traversal of the field tree rooted at T.
 */
export declare type DeepKeyOf<T extends object> = T extends (infer U)[] ? NonNullable<U> extends object ? DeepKeyOf<NonNullable<U>> : never : {
    [K in Exclude<keyof T, symbol | number>]: NonNullable<T[K]> extends object ? NonNullable<T[K]> extends ExcludedODataTypes ? K : DeepKeyOf<NonNullable<T[K]>> extends infer NextPaths extends string ? // Union this key with all the next paths separated with '/'
    K | `${K}/${NextPaths}` : K : K;
}[Exclude<keyof T, symbol | number>];
/**
 * Deeply pick fields of T using valid Cognitive Search OData $select
 * paths.
 */
export declare type DeepPick<T extends object, Keys extends DeepKeyOf<T>> = UnionToIntersection<Keys extends `${infer FieldName extends Exclude<keyof T, symbol | number>}/${infer RestPaths}` ? NonNullable<T[FieldName]> extends object ? NonNullable<T[FieldName]> extends Array<infer U extends object> ? RestPaths extends DeepKeyOf<U> ? {
    [K in FieldName]: Array<DeepPick<U, RestPaths>> | Extract<T[K], null | undefined>;
} : never : {
    [K in FieldName]: RestPaths extends DeepKeyOf<NonNullable<T[K]>> ? DeepPick<NonNullable<T[K]>, RestPaths> | Extract<T[K], null | undefined> : never;
} : never : Keys extends keyof T ? {
    [K in Keys]: T[K];
} : never> & {};
