import type { ODataSelect, ODataSelectResult } from '../lib/odata';
import { SchemaService } from './schema';
import type { AnalyzedDocument } from './analyzerService';
import { AnalyzerService } from './analyzerService';
export interface FindDocumentRequest<T extends object, Keys extends ODataSelect<T>> {
    key: string;
    select?: Keys[];
}
export interface PostDocumentsRequest<TDoc> {
    value: ({
        '@search.action'?: 'upload' | 'merge' | 'mergerOrUpload' | 'delete';
    } & TDoc)[];
}
export interface StoredDocument<T extends object> {
    key: string;
    original: T;
    analyzed: AnalyzedDocument;
}
export declare class DataStore<T extends object> {
    private readonly schemaService;
    private readonly analyzer;
    readonly documents: StoredDocument<T>[];
    constructor(schemaService: SchemaService<T>, analyzer: AnalyzerService<T>);
    private matchExisting;
    findDocument<Keys extends ODataSelect<T>>(request: FindDocumentRequest<T, Keys>): ODataSelectResult<T, Keys>;
    postDocuments(documents: PostDocumentsRequest<T>): void;
    countDocuments(): number;
    private toStoredDocument;
}
