import type { ODataSelect, ODataSelectResult } from '../lib/odata';
import type { FieldDefinition } from './schema';
import { SchemaService } from './schema';
import type { ScoringBases } from './scorer';
import type { StoredDocument } from './dataStore';
import * as Parsers from '../parsers';
export interface SearchFacetBase {
    count: number;
}
export interface SearchFacetValue extends SearchFacetBase {
    value: unknown;
}
export interface SearchFacetRange extends SearchFacetBase {
    from: unknown;
    to: unknown;
}
export declare type SearchHighlights = Record<string, FieldDefinition['type'] | string[]>;
export interface SearchFeature {
    uniqueTokenMatches: number;
    similarityScore: number;
    termFrequency: number;
}
export declare type SearchFeatures = Record<string, SearchFeature>;
export declare type SearchSuggestions = Record<string, unknown[]>;
export interface SearchDocumentMeta {
    '@search.score'?: number;
    '@search.highlights': SearchHighlights;
    '@search.features': SearchFeatures;
}
export declare type SearchResult<T extends object> = SearchDocumentMeta & T;
export interface SuggestDocumentMeta {
    '@search.text': string;
}
export interface AutocompleteResult {
    text: string;
    queryPlusText: string;
}
export declare type SuggestResult<T extends object> = SuggestDocumentMeta & T;
export interface BackendResults<T extends object> {
    '@odata.count'?: number;
    '@search.coverage'?: number;
    '@search.facets'?: Record<string, SearchFacetBase[]>;
    '@search.nextPageParameters'?: Record<string, unknown>;
    value: (SearchResult<T> | SuggestResult<T> | AutocompleteResult)[];
    '@odata.nextLink'?: string;
}
export interface ReductionResults<T extends object> {
    facets: Parsers.FacetResults;
    values: (SearchResult<T> | SuggestResult<T> | AutocompleteResult)[];
}
export declare type Reducer<T extends object, Keys extends ODataSelect<T>> = (accumulator: ReductionResults<ODataSelectResult<T, Keys>>, current: {
    document: StoredDocument<T>;
    selected?: ODataSelectResult<T, Keys>;
    globalScore: number;
    scores: ScoringBases;
    suggestions: SearchSuggestions;
    features: SearchFeatures;
}) => ReductionResults<ODataSelectResult<T, Keys>>;
export declare type DocumentMiddleware<T extends object, Keys extends ODataSelect<T>> = (next: Reducer<T, Keys>, schemaService: SchemaService<T>) => Reducer<T, Keys>;
export declare type Transformer<T extends object, Keys extends ODataSelect<T>> = (reduction: ReductionResults<ODataSelectResult<T, Keys>>, results: BackendResults<ODataSelectResult<T, Keys>>) => BackendResults<ODataSelectResult<T, Keys>>;
export declare type ResultsMiddleware<T extends object, Keys extends ODataSelect<T>> = (next: Transformer<T, Keys>, schemaService: SchemaService<T>) => Transformer<T, Keys>;
export declare class SearchBackend<T extends object> {
    private readonly schemaService;
    private readonly documentsProvider;
    constructor(schemaService: SchemaService<T>, documentsProvider: () => StoredDocument<T>[]);
    search<Keys extends ODataSelect<T>>(request: {
        documentMiddlewares: DocumentMiddleware<T, Keys>[];
        resultsMiddlewares: ResultsMiddleware<T, Keys>[];
    }): BackendResults<ODataSelectResult<T, Keys>>;
}
